@@ -1,8 +1,7 @@
 from google.cloud.bigquery import RangePartitioning, PartitionRange, TimePartitioning
-
-from pydatafabric.ye import get_spark
 from google.cloud.bigquery.job import QueryJobConfig, LoadJobConfig
 from google.cloud.exceptions import NotFound
+from pydatafabric.ye import get_spark
 
 PROJECT_ID = "emart-datafabric"
 TEMP_DATASET = "temp_1d"
@@ -48,11 +47,11 @@ def get_bigquery_storage_client(credentials=None):
     return bigquery_storage.BigQueryReadClient(credentials=credentials)
 
 
-def _get_result_schema(sql, bq_client=None):
+def _get_result_schema(sql, bq_client=None, project=PROJECT_ID):
     from google.cloud.bigquery.job import QueryJobConfig
 
     if bq_client is None:
-        bq_client = get_bigquery_client()
+        bq_client = get_bigquery_client(project=project)
     job_config = QueryJobConfig(
         dry_run=True,
         use_query_cache=False,
@@ -80,15 +79,15 @@ def _print_query_job_results(query_job):
             f"query: {query_job.query}\n"
             f"destination: {dest_str}\n"
             f"total_rows: {query_job.result().total_rows}\n"
-            f"slot_secs: {query_job.slot_millis/1000}\n"
+            f"slot_secs: {query_job.slot_millis / 1000}\n"
         )
     except Exception as e:
         print("Warning: exception on print statistics")
         print(e)
 
 
-def bq_insert_overwrite_table(sql, destination):
-    bq = get_bigquery_client()
+def bq_insert_overwrite_table(sql, destination, project=PROJECT_ID):
+    bq = get_bigquery_client(project=project)
     table = bq.get_table(destination)
     if table.time_partitioning or table.range_partitioning:
         load_query_result_to_partitions(sql, destination)
@@ -105,13 +104,19 @@ def bq_insert_overwrite_table(sql, destination):
         bq.close()
 
 
-def bq_ctas(sql, destination=None, partition_by=None, clustering_fields=None):
+def bq_ctas(
+    sql,
+    destination=None,
+    project=PROJECT_ID,
+    partition_by=None,
+    clustering_fields=None,
+):
     """
     create new table and insert results
     """
     from google.cloud.bigquery.job import QueryJobConfig
 
-    bq = get_bigquery_client()
+    bq = get_bigquery_client(project=project)
     if partition_by:
         partition_type = _get_result_column_type(sql, partition_by, bq_client=bq)
         if partition_type == "DATE":
@@ -153,14 +158,14 @@ def bq_ctas(sql, destination=None, partition_by=None, clustering_fields=None):
     return job.destination
 
 
-def _bq_query_to_new_table(sql, destination=None):
-    return bq_ctas(sql, destination)
+def _bq_query_to_new_table(sql, destination=None, project=PROJECT_ID):
+    return bq_ctas(sql, destination, project=project)
 
 
-def _bq_query_to_existing_table(sql, destination):
+def _bq_query_to_existing_table(sql, destination, project=PROJECT_ID):
     from google.cloud.bigquery.job import QueryJobConfig
 
-    bq = get_bigquery_client()
+    bq = get_bigquery_client(project=project)
     table = bq.get_table(destination)
     config = QueryJobConfig(
         destination=destination,
@@ -177,9 +182,9 @@ def _bq_query_to_existing_table(sql, destination):
     return job.destination
 
 
-def _bq_table_to_pandas(table):
+def _bq_table_to_pandas(table, project=PROJECT_ID):
     credentials = get_credentials()
-    bq = get_bigquery_client(credentials=credentials)
+    bq = get_bigquery_client(credentials=credentials, project=project)
     bqstorage_client = get_bigquery_storage_client(credentials=credentials)
     row_iterator = bq.list_rows(table)
     df = row_iterator.to_dataframe(
@@ -268,33 +273,33 @@ def load_bigquery_ipython_magic():
         )
 
 
-def get_bigquery_client(credentials=None, project_id=PROJECT_ID):
+def get_bigquery_client(credentials=None, project=PROJECT_ID):
     from google.cloud import bigquery
 
     if credentials is None:
         credentials = get_credentials()
 
-    return bigquery.Client(credentials=credentials, project=project_id)
+    return bigquery.Client(credentials=credentials, project=project)
 
 
-def bq_table_exists(table):
+def bq_table_exists(table, project=PROJECT_ID):
     try:
-        get_bigquery_client().get_table(table)
+        get_bigquery_client(project=project).get_table(table)
     except NotFound:
         return False
     return True
 
 
-def get_unhidden_partitions(table_name):
+def get_unhidden_partitions(table_name, project=PROJECT_ID):
     parts = filter(
         lambda x: x not in ["__NULL__", "__UNPARTITIONED__"],
-        get_bigquery_client().list_partitions(table_name),
+        get_bigquery_client(project=project).list_partitions(table_name),
     )
     return parts
 
 
-def _get_partition_filter(dataset, table_name, partition):
-    table = get_bigquery_client().get_table(f"{dataset}.{table_name}")
+def _get_partition_filter(dataset, table_name, partition, project=PROJECT_ID):
+    table = get_bigquery_client(project=project).get_table(f"{dataset}.{table_name}")
     if "timePartitioning" in table._properties:
         partition_column_name = table._properties["timePartitioning"]["field"]
         return f"{partition_column_name} = '{partition}'"
@@ -313,7 +318,13 @@ def bq_to_pandas(sql, large=False):
 
 
 def _bq_table_to_df(
-    dataset, table_name, col_list, partition=None, where=None, spark_session=None
+    dataset,
+    table_name,
+    col_list,
+    project=PROJECT_ID,
+    partition=None,
+    where=None,
+    spark_session=None,
 ):
     import base64
     from pydatafabric.vault_utils import get_secrets
@@ -324,12 +335,14 @@ def _bq_table_to_df(
     key = get_secrets(CREDENTIALS_SECRET_MOUNT_POINT, CREDENTIALS_SECRET_PATH)["config"]
     df = (
         spark_session.read.format("bigquery")
-        .option("project", PROJECT_ID)
-        .option("table", f"{PROJECT_ID}:{dataset}.{table_name}")
+        .option("project", project)
+        .option("table", f"{project}:{dataset}.{table_name}")
         .option("credentials", base64.b64encode(key.encode()).decode())
     )
     if partition:
-        table = get_bigquery_client().get_table(f"{dataset}.{table_name}")
+        table = get_bigquery_client(project=project).get_table(
+            f"{dataset}.{table_name}"
+        )
         if "timePartitioning" in table._properties:
             partition_column_name = table._properties["timePartitioning"]["field"]
             filter = f"{partition_column_name} = '{partition}'"
@@ -347,10 +360,16 @@ def _bq_table_to_df(
 
 
 def bq_table_to_df(
-    dataset, table_name, col_list, partition=None, where=None, spark_session=None
+    dataset,
+    table_name,
+    col_list,
+    project=PROJECT_ID,
+    partition=None,
+    where=None,
+    spark_session=None,
 ):
     return _bq_table_to_df(
-        dataset, table_name, col_list, partition, where, spark_session
+        dataset, table_name, col_list, project, partition, where, spark_session
     )
 
 
@@ -359,18 +378,21 @@ def bq_table_to_parquet(
     table_name,
     output_dir,
     col_list="*",
+    project=PROJECT_ID,
     partition=None,
     where=None,
     mode="overwrite",
 ):
     bq_table_to_df(
-        dataset, table_name, col_list, partition=partition, where=where
+        dataset, table_name, col_list, project=project, partition=partition, where=where
     ).write.mode(mode).parquet(output_dir)
 
 
-def bq_table_to_pandas(dataset, table_name, col_list="*", partition=None, where=None):
+def bq_table_to_pandas(
+    dataset, table_name, col_list="*", project=PROJECT_ID, partition=None, where=None
+):
     col_list_str = ", ".join(col_list) if isinstance(col_list, list) else col_list
-    query = f"SELECT {col_list_str} FROM {dataset}.{table_name} "
+    query = f"SELECT {col_list_str} FROM {project}.{dataset}.{table_name} "
     if partition or where:
         query += "WHERE "
         conditions = []
@@ -388,6 +410,7 @@ def _df_to_bq_table(
     df,
     dataset,
     table_name,
+    project=PROJECT_ID,
     partition=None,
     partition_field=None,
     clustering_fields=None,
@@ -404,7 +427,7 @@ def _df_to_bq_table(
     )
     df = (
         df.write.format("bigquery")
-        .option("project", PROJECT_ID)
+        .option("project", project)
         .option("credentials", base64.b64encode(key.encode()).decode())
         .option("table", table)
         .option("temporaryGcsBucket", "emart-datafabric-temp-7d")
@@ -420,13 +443,21 @@ def df_to_bq_table(
     df,
     dataset,
     table_name,
+    project=PROJECT_ID,
     partition=None,
     partition_field=None,
     clustering_fields=None,
     mode="overwrite",
 ):
     _df_to_bq_table(
-        df, dataset, table_name, partition, partition_field, clustering_fields, mode
+        df,
+        dataset,
+        table_name,
+        project,
+        partition,
+        partition_field,
+        clustering_fields,
+        mode,
     )
 
 
@@ -434,6 +465,7 @@ def parquet_to_bq_table(
     parquet_dir,
     dataset,
     table_name,
+    project=PROJECT_ID,
     partition=None,
     partition_field=None,
     clustering_fields=None,
@@ -443,7 +475,14 @@ def parquet_to_bq_table(
         spark = get_spark()
         df = spark.read.format("parquet").load(parquet_dir)
         _df_to_bq_table(
-            df, dataset, table_name, partition, partition_field, clustering_fields, mode
+            df,
+            dataset,
+            table_name,
+            project,
+            partition,
+            partition_field,
+            clustering_fields,
+            mode,
         )
     finally:
         spark.stop()
@@ -453,6 +492,7 @@ def pandas_to_bq_table(
     pd_df,
     dataset,
     table_name,
+    project=PROJECT_ID,
     partition=None,
     partition_field=None,
     clustering_fields=None,
@@ -465,6 +505,7 @@ def pandas_to_bq_table(
             spark_df,
             dataset,
             table_name,
+            project,
             partition,
             partition_field,
             clustering_fields,
@@ -475,11 +516,16 @@ def pandas_to_bq_table(
 
 
 def pandas_to_bq(
-    pd_df, destination, partition=None, clustering_fields=None, overwrite=True
+    pd_df,
+    destination,
+    project=PROJECT_ID,
+    partition=None,
+    clustering_fields=None,
+    overwrite=True,
 ):
     range_partitioning = None
     time_partitioning = None
-    bq = get_bigquery_client()
+    bq = get_bigquery_client(project=project)
     if bq_table_exists(destination):
         target_table = bq.get_table(destination)
         range_partitioning = target_table.range_partitioning
@@ -544,14 +590,14 @@ def rdd_to_pandas(func):
     return _rdd_to_pandas
 
 
-def bq_to_df(query, spark_session=None):
+def bq_to_df(query, project=PROJECT_ID, spark_session=None):
     temp_table_name = get_temp_table()
     jc = QueryJobConfig(
         create_disposition="CREATE_IF_NEEDED",
         write_disposition="WRITE_TRUNCATE",
         destination=temp_table_name,
     )
-    bq_client = get_bigquery_client()
+    bq_client = get_bigquery_client(project=project)
     job = bq_client.query(query, job_config=jc)
     job.result()
     t = temp_table_name.split(".")
@@ -560,9 +606,9 @@ def bq_to_df(query, spark_session=None):
 
 
 def load_query_result_to_table(
-    dest_table, query, part_col_name=None, clustering_fields=None
+    dest_table, query, project=PROJECT_ID, part_col_name=None, clustering_fields=None
 ):
-    bq_client = get_bigquery_client()
+    bq_client = get_bigquery_client(project=project)
     print(query)
     if bq_table_exists(dest_table):
         table = bq_client.get_table(dest_table)
@@ -619,11 +665,11 @@ def load_query_result_to_table(
         bq_client.query(f"SELECT * FROM {temp_table_name}", job_config=qjc).result()
 
 
-def get_temp_table():
+def get_temp_table(project=PROJECT_ID):
     import uuid
 
     table_id = str(uuid.uuid4()).replace("-", "_")
-    full_table_id = f"{PROJECT_ID}.{TEMP_DATASET}.{table_id}"
+    full_table_id = f"{project}.{TEMP_DATASET}.{table_id}"
 
     return full_table_id
 
@@ -637,11 +683,11 @@ def _get_partition_name(table):
         raise Exception(f"Table[{table}] is not partitioned.")
 
 
-def load_query_result_to_partitions(query, dest_table):
+def load_query_result_to_partitions(query, dest_table, project=PROJECT_ID):
     from google.cloud.bigquery.table import TableReference
     from google.cloud.bigquery.dataset import DatasetReference
 
-    bq = get_bigquery_client()
+    bq = get_bigquery_client(project=project)
     table = bq.get_table(dest_table)
 
     """
@@ -698,9 +744,9 @@ def load_query_result_to_partitions(query, dest_table):
             job.result()
             _print_query_job_results(job)
         else:
-            project_id, dataset_id, table_id = dest_table.split(".")
+            project, dataset_id, table_id = dest_table.split(".")
             ref = TableReference(
-                DatasetReference(project_id, dataset_id), f"{table_id}${p}"
+                DatasetReference(project, dataset_id), f"{table_id}${p}"
             )
             qjc = QueryJobConfig(
                 destination=ref,
@@ -722,10 +768,10 @@ def load_query_result_to_partitions(query, dest_table):
     return partitions
 
 
-def get_max_part(table_name):
+def get_max_part(table_name, project=PROJECT_ID):
     from datetime import datetime
 
-    bq_client = get_bigquery_client()
+    bq_client = get_bigquery_client(project=project)
     parts = get_unhidden_partitions(table_name)
 
     if not parts:
@@ -743,26 +789,36 @@ def get_max_part(table_name):
 
 
 def bq_insert_overwrite(
-    sql, destination, suffixes=None, partition=None, clustering_fields=None
+    sql,
+    destination,
+    suffixes=None,
+    project=PROJECT_ID,
+    partition=None,
+    clustering_fields=None,
 ):
     if suffixes:
         bq_insert_overwrite_with_suffixes(
-            sql, destination, suffixes, partition, clustering_fields
+            sql, destination, suffixes, project, partition, clustering_fields
         )
     else:
         bq_insert_overwrite_without_suffixes(
-            sql, destination, partition, clustering_fields
+            sql, destination, project, partition, clustering_fields
         )
 
 
 def bq_insert_overwrite_with_suffixes(
-    sql, destination, suffixes, partition=None, clustering_fields=None
+    sql,
+    destination,
+    suffixes,
+    project=PROJECT_ID,
+    partition=None,
+    clustering_fields=None,
 ):
     temp_table = get_temp_table()
     bq_ctas(
         sql, temp_table, partition_by=partition, clustering_fields=clustering_fields
     )
-    bq = get_bigquery_client()
+    bq = get_bigquery_client(project=project)
     r = bq.query(f"""select distinct {', '.join(suffixes)} from {temp_table}""")
     for cols in r:
         suffix = "".join([f"__{col}" for col in cols])
@@ -771,16 +827,20 @@ def bq_insert_overwrite_with_suffixes(
         target = "".join([destination, suffix])
         sub_sql = f"{select_clause} where {where_clause}"
         bq_insert_overwrite_without_suffixes(
-            sub_sql, target, partition=partition, clustering_fields=clustering_fields
+            sub_sql,
+            target,
+            project=project,
+            partition=partition,
+            clustering_fields=clustering_fields,
         )
     bq.close()
 
 
 def bq_insert_overwrite_without_suffixes(
-    sql, destination, partition=None, clustering_fields=None
+    sql, destination, project=PROJECT_ID, partition=None, clustering_fields=None
 ):
     if bq_table_exists(destination):
-        bq_insert_overwrite_table(sql, destination)
+        bq_insert_overwrite_table(sql, destination, project=project)
     else:
         bq_ctas(
             sql,
